/*******************************************************************************
 * (c) Copyright 2007-2008 Actel Corporation.  All rights reserved.
 * 
 * CoreI2C software driver Application Programming Interface.
 * This file contains defines and function declarations allowing to interface
 * with the CoreI2C software driver.
 * 
 * SVN $Revision: 567 $
 * SVN $Date: 2008-11-18 12:35:04 +0000 (Tue, 18 Nov 2008) $
 */
#ifndef CORE_I2C_H_
#define CORE_I2C_H_

#include "cpu_types.h"

/*------------------------------------------------------------------------------
 * Serial clock selection.
 * The I2C serial clock frequency is the APB bus clock speed divided down.
 * The possible divider are listed down.
 */
typedef enum i2c_clock_divider {
    PCLK_DIV_256 = 0,
    PCLK_DIV_224,
    PCLK_DIV_192,
    PCLK_DIV_160,
    PCLK_DIV_960,
    PCLK_DIV_120,
    PCLK_DIV_60,
    BCLK_DIV_8
} i2c_clock_divider_t;

/*------------------------------------------------------------------------------
 * Options
 */
#define RELEASE_BUS     0x00
#define HOLD_BUS        0x01
 
/*------------------------------------------------------------------------------
 * I2C transaction status:
 */
typedef enum i2c_status
{
    SUCCESS = 0,
    IN_PROGRESS,
    FAILED
} i2c_status_t;

/*------------------------------------------------------------------------------
 * Slave write handler return values:
 */
typedef enum slave_handler_ret {
    REENABLE_SLAVE_RX = 0,
    PAUSE_SLAVE_RX = 1
} i2c_slave_handler_ret_t;

/*------------------------------------------------------------------------------
 * Slave write handler functions prototype.
 *------------------------------------------------------------------------------ 
 * This defines the function prototype that must be followed by I2C slave write
 * handler functions. These functions are registered with the I2C driver through
 * the I2C_register_write_handler() function.
 * 
 * Slave write handler functions should follow the following prototype:
 * 		i2c_slave_handler_ret_t write_handler( uint8_t * data, uint16_t size );
 * 
 * The first parameter is a pointer to a buffer holding the data written to the
 * I2C slave. The second parameter is the number of bytes held in the received
 * data buffer.
 * 
 * Handler functions must return either REENABLE_SLAVE_RX or PAUSE_SLAVE_RX.
 * This either (REENABLE_SLAVE_RX) cause the received data buffer to be released
 * and allows another I2C write transactions to take place, or (PAUSE_SLAVE_RX)
 * causes subsequent write requests to be NACKed by the slave until the received
 * buffer content has been processed by some other part of the software
 * application. A call to I2C_enable_slave_rx() is required at some point after
 * returning PAUSE_SLAVE_RX in order to release the rx data buffer so it can be
 * used to store data received by subsequent I2C write transactions.
 */
typedef i2c_slave_handler_ret_t (*i2c_slave_wr_handler_t)( uint8_t *, uint16_t );

/*------------------------------------------------------------------------------
 * i2c_instance_t
 *------------------------------------------------------------------------------
 * The content of this data structure should not be modified or used outside of
 * the CoreI2C driver. Software using the CoreI2C driver should only need to 
 * create one single instance of this data structure for each CoreI2C hardware
 * instance in the system then pass a pointer to these data structures with each
 * call to the CoreI2C driver in order to identify the CoreI2C hardware instance
 * it wishes to use.
 */
typedef struct i2c_instance
{
	addr_t base_address;
	uint_fast8_t ser_address;
	/* Transmit related info:*/
	uint_fast8_t target_addr;
	
	/* Current transaction type (WRITE, READ, RANDOM_READ)*/
	uint8_t transaction;
	
	uint_fast16_t random_read_addr;

    uint8_t options;
	
	/* TX INFO: */
	uint8_t * tx_buffer;
	uint_fast16_t tx_size;
	uint_fast16_t tx_idx;
	uint_fast8_t dir;
	
	/* RX INFO: */
	uint8_t * rx_buffer;
	uint_fast16_t rx_size;
	uint_fast16_t rx_idx;	
	
	/* status variable: */
    volatile i2c_status_t status;
	
	/* Slave data: */
	uint_fast8_t slave_mem_offset_length;
	i2c_slave_wr_handler_t slave_write_handler;
    
} i2c_instance_t;

/*------------------------------------------------------------------------------
 * CoreI2C initialisation routine.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	Initialise a CoreI2C instance hardware and data structures.
 * 
 *------------------------------------------------------------------------------ 
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the i2c_instance_t data structure holding all data related to
 * 		the CoreI2C instance being initialised. A pointer to the same data
 * 		structure will be used in subsequent calls to the CoreI2C driver functions
 * 		in order to identify the CoreI2C instance that should perform the
 * 		operation implemented by the called driver function.
 *   
 * 	addr_t base_address:
 * 		Base address in the processor's memory map of the CoreI2C instance being
 * 		initialised.
 * 
 * 	uint8_t ser_address:
 * 		This parameter sets the I2C serial address being initialized. It is the I2C
 *		bus address to which the CoreI2C instance will respond.
 *  
 * 	uint8_t ser_clock_speed:
 *		This parameter sets the I2C serial clock frequency. It selects the divider
 *		that will be used to generate the serial clock from the APB clock. 		 
 */
void I2C_init
(
	i2c_instance_t * this_i2c,
	addr_t base_address,
	uint8_t ser_address,
	i2c_clock_divider_t ser_clock_speed
);

/*------------------------------------------------------------------------------
 * CoreI2C interrupt service routine.
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the i2c_instance_t data structure holding all data related to
 * 		the	CoreI2C instance that generated the interrupt.
 */
void I2C_isr
(
	i2c_instance_t * this_i2c
);

/*******************************************************************************
 *******************************************************************************
 * 
 *                           Master specific functions
 * 
 * The following functions are only used within an I2C master's implementation.
 */
 
/*------------------------------------------------------------------------------
 * I2C master write function.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	This function initiates an I2C master write transaction. This function
 *	returns immediately after initiating the transaction. The content of the
 *	write buffer passed as parameter should not be modified until the write
 *	transaction completes. It also means that the memory allocated for the write
 *	buffer should not be freed or go out of scope before the write completes.
 *	You can check for the write transaction completion using the I2C_status()
 *	function.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the i2c_instance_t data structure holding all data related to
 * 		the	CoreI2C instance that will initiate an I2C write transaction as a
 * 		result to a call to this function.
 * 
 * 	uint8_t serial_addr:
 * 		This parameter specifies the serial address of the target I2C device.
 * 
 * 	uint8_t * write_buffer:
 * 		This parameter is a pointer to a buffer holding the data to be written
 *		to the target I2C device.
 * 		Care must be taken in not releasing the memory used by this buffer before
 *		the write transaction completes. For example, it is not appropriate to
 * 		return from a function allocating this buffer as an array variable before
 * 		the write transaction completes as this would result in the buffer's
 * 		memory being de-allocated from the stack when the function returns. This 
 * 		memory could then be subsequently reused and modified causing unexpected
 * 		data to be written to the target I2C device. 
 * 
 * 	uint16_t write_size:
 * 		Number of bytes held in the write_buffer to be written to the target I2C
 * 		device.
 *
 *	uint8_t options:
 *		The options parameter is used to indicate if the I2C bus should be released
 *		on completion of the write transaction. Using the RELEASE_BUS constant for
 *		the options parameter causes a STOP bit to be generated at the end of the
 *		write transaction causing the bus to be released for other I2C devices to
 *		use. Using the HOLD_BUS constant as options parameter prevents a STOP bit
 *		from being generated at the end of the write transaction, preventing other
 *		I2C devices from initiating a bus transaction.
 */
void I2C_write
(
	i2c_instance_t * this_i2c,
	uint8_t serial_addr,
	uint8_t * write_buffer,
	uint16_t write_size,
    uint8_t options
);

/*------------------------------------------------------------------------------
 * I2C master read.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	Initiates an I2C master read transaction. This function returns immediatly
 * 	after initiating the transaction.
 * 	You can check for the read transaction completion using the 
 * 	I2C_read_status() function.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the i2c_instance_t data structure holding all data related to
 * 		the	CoreI2C instance that will initiate an I2C read transaction as a
 * 		result to a call to this function.
 * 
 * 	uint8_t serial_addr:
 * 		This parameter specifies the serial address of the target I2C device.
 * 
 * 	uint8_t * read_buffer
 * 		Pointer to a buffer where the data received from the target device will
 * 		be stored.
 * 		Care must be taken in not releasing the memory used by this buffer before
 *		the read transaction completes. For example, it is not appropriate to
 * 		return from a function allocating this buffer as an array variable before
 * 		the read transaction completes as this would result in the buffer's
 * 		memory being de-allocated from the stack when the function returns. This 
 * 		memory could then be subsequently reallocated resulting in the read
 * 		transaction corrupting the newly allocated memory.
 *  
 * 	uint16_t read_size:
 * 		This parameter is the number of bytes to read from the target device. This
 *		size must not exceed the size of the read_buffer buffer.
 *
 *  uint8_t options:
 *		The options parameter is used to indicate if the I2C bus should be released
 *		on completion of the read transaction. Using the RELEASE_BUS constant for
 *		the options parameter causes a STOP bit to be generated at the end of the
 *		read transaction causing the bus to be released for other I2C devices to use.
 *		Using the HOLD_BUS constant as options parameter prevents a STOP bit from
 *		being generated at the end of the read transaction, preventing other I2C
 *		devices from initiating a bus transaction.
 */
void I2C_read
(
	i2c_instance_t * this_i2c,
	uint8_t serial_addr,
	uint8_t * read_buffer,
	uint16_t read_size,
    uint8_t options
);

/*------------------------------------------------------------------------------
 * I2C master write-read
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	Initiates an I2C write-read transaction where data is first written to the
 *  target device before issuing a restart condition and changing the direction
 *  of the I2C transaction in order to read from the target device.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the i2c_instance_t data structure holding all data related to
 * 		the	CoreI2C instance that will initiate an I2C write-read transaction
 * 		as a result to a call to this function.
 * 
 * 	uint8_t serial_addr:
 * 		This parameter specifies the serial address of the target I2C device.
 * 
 * 	uint8_t * addr_offset:
 * 		This parameter is a pointer to the buffer containing the data that will
 *		be sent to the slave during the write phase of the write-read
 *		transaction. This data is typically used to specify an address offset
 *		specifying to the I2C slave device what data is must return during the
 *		read phase of the write-read transaction.
 * 
 * 	uint16_t offset_size:
 * 		This parameter specifies the number of offset bytes to be written
 *		during the write phase of the write-read transaction. This is typically
 *		the size of the buffer pointed to by the addr_offset parameter.
 * 
 * 	uint8_t * read_buffer:
 * 		This parameter is a pointer to the buffer where the data read from the
 *		I2C slave will be stored.
 * 
 * 	uint16_t read_size:
 * 		This parameter specifies the number of bytes to read from the target I2C
 *		slave device. This size must not exceed the size of the buffer pointed
 *		to by the read_buffer parameter.
 *
 *  uint8_t options:
 *		The options parameter is used to indicate if the I2C bus should be released
 *		on completion of the write-read transaction. Using the RELEASE_BUS constant
 *		for the options parameter causes a STOP bit to be generated at the end of
 *		the write-read transaction causing the bus to be released for other I2C
 *		devices to use. Using the HOLD_BUS constant as options parameter prevents
 *		a STOP bit from being generated at the end of the write-read transaction,
 *		preventing other I2C devices from initiating a bus transaction.
 */
void I2C_write_read
(
	i2c_instance_t * this_i2c,
	uint8_t serial_addr,
	uint8_t * addr_offset,
	uint16_t offset_size,
	uint8_t * read_buffer,
	uint16_t read_size,
    uint8_t options
);
	
/*------------------------------------------------------------------------------
 * I2C status
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	This function indicates the current state of a CoreI2C instance.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 	    Pointer to the i2c_instance_t data structure holding all data related to
 * 	    the	CoreI2C instance we are inquiring the status from.
 * 
 *------------------------------------------------------------------------------
 * Return values:
 * 
 * 	The return value indicates the current state of a CoreI2C instance or the
 *  outcome of the previous transaction if no transaction is in progress.
 *  Possible return values are: 
 *      SUCCESS:
 *          The last I2C transaction has completed successfully.
 *      IN_PROGRESS:
 *          There is a an I2C transaction in progress.
 *      FAILED:
 *          The last I2C transaction failed.
 */
i2c_status_t I2C_get_status
(
    i2c_instance_t * this_i2c
);

/*------------------------------------------------------------------------------
 * Wait for I2C transaction completion.
 *------------------------------------------------------------------------------
 * Description:
 * 
 *  This function waits for the current I2C transaction to complete. The return
 *  value indicates whether the last I2C transaction was successful or failed.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 *  i2c_instance_t * this_i2c:
 *      Pointer to the i2c_instance_t data structure holding all data related to
 *      the CoreI2C instance we are waiting on.
 * 
 *------------------------------------------------------------------------------
 * Return values:
 * 
 *  The return value indicates the outcome of the last I2C transaction. It can
 *  be one of the following: 
 *      SUCCESS:
 *          The last I2C transaction has completed successfully.
 *      FAILED:
 *          The last I2C transaction failed.
 * 
 */
i2c_status_t I2C_wait_complete
(
    i2c_instance_t * this_i2c
);


/*******************************************************************************
 *******************************************************************************
 * 
 *                           Slave specific functions
 * 
 * The following functions are only used within the implementation of an I2C
 * slave device.
 */

/*------------------------------------------------------------------------------
 * I2C slave transmit buffer configuration.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	This function specifies the memory buffer holding the data that will be sent
 * 	to the I2C master when this instance is the target of a read or write-read
 * 	transaction.
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the data structure holding all information related to the I2C
 * 		instance which is to be configured through this function call.
 * 
 * 	uint8_t * tx_buffer:
 * 		This parameter is a pointer to the memory holding the data to be returned
 *		to the I2C master when this I2C instance is the target of a read or
 *		write-read transaction.
 * 
 * 	uint16_t tx_size:
 * 		Size of the transmit buffer.
 */
void I2C_set_slave_tx_buffer
(
	i2c_instance_t * this_i2c,
	uint8_t * tx_buffer,
	uint16_t tx_size
);

/*------------------------------------------------------------------------------
 * I2C slave receive buffer configuration.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	This function specifies the memory buffer that will be used by the I2C
 * 	instance to receive data when it is a slave. This buffer is the memory where
 * 	data will be stored when the device is the target of a master write
 * 	transaction (i.e. when it is the slave).
 * 	
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the data structure holding all information associated with
 * 		the	I2C instance for which we are configuring the slave receive buffer.
 * 		This data structure is used to identify the I2C instance we are 
 * 		configuring.
 * 
 * 	uint8_t * rx_buffer:
 * 		Pointer to the memory allocated by tha caller to be used as slave
 * 		receive buffer.
 * 
 * 	uint16_t rx_size:
 * 		Size of the slave receive buffer. This is the amount of memory that is
 * 		allocated to the buffer pointed to by "rx_buffer".
 * 		Please note that the buffer size will indirectly specify the maximum
 * 		I2C write transaction length this I2C instance can be the target of.
 * 		This is because this I2C instance will NACK received bytes as soon as
 * 		its received buffer is full causing the write transaction it is the
 * 		target of to fail.   
 */
void I2C_set_slave_rx_buffer
(
	i2c_instance_t * this_i2c,
	uint8_t * rx_buffer,
	uint16_t rx_size
);

/*------------------------------------------------------------------------------
 * I2C slave memory offset length configuration.
 *------------------------------------------------------------------------------
 * Description:
 * 	
 * 	This function configures the number of bytes to be interpreted as memory
 *	offset in slave mode during write-read transactions.
 *   
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the data structure associated with the I2C instance being
 * 		configured. This data structure is used to identify the I2C instance
 * 		configured through this function call.
 * 
 * 	uint8_t offset_length:
 * 		This parameter contains the number of bytes that will be interpreted as
 *		memory offset during read and write-read transactions when this I2C
 *		instance is the slave.
 */
void I2C_set_slave_mem_offset_length
(
	i2c_instance_t * this_i2c,
	uint8_t offset_length
);

/*------------------------------------------------------------------------------
 * I2C write handler registration. 
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	Register the function that will be called to process the data written to
 * 	this I2C instance when it is the slave in an I2C write transaction. 
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the data structure associated with the I2C instance being 
 * 		configured. This data structure is used to identify the I2C instance
 * 		configured through this function call.
 * 
 * 	slave_wr_handler_fct_t handler:
 * 		Pointer to the function that will process the I2C write request.
 * 		See the comments about the slave_wr_handler_fct_t typedef above for
 * 		details about the handler function prototype.
 */
void I2C_register_write_handler
(
	i2c_instance_t * this_i2c,
	i2c_slave_wr_handler_t handler
);

/*------------------------------------------------------------------------------
 * I2C slave receive enable.
 *------------------------------------------------------------------------------
 * Description:
 * 
 * 	Enables the I2C instance identified through the "this_i2c" parameter to
 * 	receive data when it is the target of an I2C write transaction.
 * 
 *------------------------------------------------------------------------------
 * Parameters:
 * 
 * 	i2c_instance_t * this_i2c:
 * 		Pointer to the data structure holding all information related to the 
 * 		I2C instance which is enabled to receive. 
 */
void I2C_enable_slave_rx
(
	i2c_instance_t * this_i2c
);

#endif /*CORE_I2C_H_*/
